@@ -29,6 +29,8 @@ def base_parser():
     * --basedir: base directory. (default=os.getcwd)
     * --input-encoding: input data encoding. (default=utf-8)
     * --output-encoding: output data encoding. (default=utf-8)
+    * --processes: count of processes.
+    * --chunksize: a number of chunks submitted to the process pool.
     """
     parser = argparse.ArgumentParser(add_help=False)
 
@@ -55,6 +57,12 @@ def base_parser():
                 default=DEFAULT_ENCODING,
                 help="encoding of output distination")
 
+    parser.add_argument("--processes", dest="processes", type=int,
+                help="count of processes")
+
+    parser.add_argument("--chunksize", dest="chunksize", type=int,
+                help="a number of chunks submitted to the process pool")
+
     group = parser.add_mutually_exclusive_group()
 
     group.add_argument("-v", "--verbose", dest="verbose",
@@ -86,6 +94,9 @@ def parse_arguments(*args, **kwargs):
 
     if args.verbose:
         logging.basicConfig(level=logging.DEBUG)
+        if args.processes:
+            import multiprocessing.util
+            multiprocessing.util.log_to_stderr(logging.DEBUG)
     elif not args.quiet:
         logging.basicConfig(level=logging.INFO)
 
