@@ -4,9 +4,10 @@
 """ Stream processing utility.
 """
 
-import logging
 import gzip
 import json
+import logging
+import multiprocessing
 import os
 import sys
 import time
@@ -22,6 +23,7 @@ from clitool import (
     PROCESSING_TIME
 )
 
+
 if sys.version_info.major == 3:
     import csv
     import io
@@ -30,8 +32,10 @@ if sys.version_info.major == 3:
         return csv.reader(io.TextIOWrapper(fp.buffer, encoding))
 
     csvreader = csvreader3
+    imap = map  # backward compat
 
 else:
+    from itertools import imap
     from ._unicodecsv import UnicodeReader
 
     def csvreader2(fp, encoding):
@@ -63,7 +67,7 @@ class SimpleDictReporter(object):
 
     def report(self):
         """
-        :rtype: dictionary
+        :rtype: dict
         """
         return dict(self.counter)
 
@@ -99,6 +103,43 @@ class RowMapper(object):
             len(self.header), len(row))
 
 
+class Functions(object):
+
+    """
+    :param item: item to process.
+    :rtype: True if skipped, False if error, and valid object if processed.
+    """
+
+    def __init__(self, *args):
+        self.procedures = tuple(filter(lambda r: r, args))
+        logging.debug("%d procedures are set.", len(self.procedures))
+        self.reporting_interval = PROCESSING_REPORTING_INTERVAL
+
+    def __call__(self, tup):
+        i, item = tup
+        if i % self.reporting_interval == 0:
+            logging.info("Processing %dth item.", i)
+        if not item:  # Skip
+            return True
+        ret = item
+        for proc in self.procedures:
+            try:
+                ret = proc(ret)
+            except KeyboardInterrupt:
+                logging.info("Stopped by user interruption at %dth item.", i)
+                raise SystemExit("Stopped by user interruption.")
+            except:
+                logging.error("Fail to process at %dth item.", i)
+                return None
+            if ret is None:  # Skip
+                return True
+            elif not ret:  # Error
+                return False
+        return ret
+
+
+# Can not recycle pool . . .
+# http://stackoverflow.com/questions/5481104/multiprocessing-pool-imap-broken
 class Streamer(object):
 
     """ Simple streaming module to accept step-by-step procedures.
@@ -119,54 +160,24 @@ class Streamer(object):
 
     :param callback: function to collect parsed value
     :type callback: callable
-    :param args: procedures
-    :type args: list of callbables
+    :param args: callables
+    :type args: list
     """
 
     def __init__(self, callback=None, *args, **kwargs):
-        self.procedures = tuple(filter(lambda r: r, args))
+        self.func = Functions(*args)
         self.collect = callback or (lambda r: r)
-        self.reporting_interval = PROCESSING_REPORTING_INTERVAL
-        logging.debug("%d procedures are set.", len(self.procedures))
+        self.processes = kwargs.get('processes')
+        if self.processes and self.processes > multiprocessing.cpu_count():
+                logging.warn("given processes is %d, count of CPU is %d" % (
+                    self.processes, multiprocessing.cpu_count()))
 
-    def process_stream_item(self, item):
-        """
-        :param item: item to process.
-        :rtype: True if skipped, False if error, and valid object if processed.
-        """
-        if not item:  # Skip
-            return True
-        ret = item
-        for proc in self.procedures:
-            ret = proc(ret)
-            if ret is None:  # Skip
-                return True
-            elif not ret:  # Error
-                return False
-        self.collect(ret)
-        return ret
+    def consume(self, stream, chunksize=1):
+        """ Consuming given strem object and returns processing stats.
 
-    def iterate(self, stream):
-        """ Generator interface to allow iterable process.
-        """
-        for i, raw in enumerate(stream, 1):
-            try:
-                processed = self.process_stream_item(raw)
-            except KeyboardInterrupt:
-                logging.info("Stopped by user interruption at %dth item.", i)
-                raise SystemExit("Stopped by user interruption.")
-            except:
-                processed = None
-                logging.error("Fail to process at %dth item.", i)
-            yield processed
-            if i % self.reporting_interval == 0:
-                logging.info("Processed %dth item.", i)
-
-    def consume(self, stream):
-        """
         :param stream: streaming object to consume
         :type stream: iterable
-        :rtype: dictionary stats after processing.
+        :rtype: dict
         """
         stats = {
             PROCESSING_TOTAL: 0,
@@ -175,17 +186,27 @@ class Streamer(object):
             PROCESSING_ERROR: 0,
             PROCESSING_FAILURE: 0
         }
+        if self.processes:
+            pool = multiprocessing.Pool(processes=self.processes)
+            results = pool.imap_unordered(self.func,
+                        enumerate(stream, 1), chunksize=chunksize)
+        else:
+            results = imap(self.func, enumerate(stream, 1))
         start = time.time()
-        for processed in self.iterate(stream):
+        for processed in results:
             if processed is True:
                 stats[PROCESSING_SKIPPED] += 1
             elif processed:
                 stats[PROCESSING_SUCCESS] += 1
+                self.collect(processed)
             elif processed is None:
                 stats[PROCESSING_FAILURE] += 1
             else:
                 stats[PROCESSING_ERROR] += 1
             stats[PROCESSING_TOTAL] += 1
+        if self.processes:
+            pool.close()
+            pool.join()
         stats[PROCESSING_TIME] = time.time() - start
         logging.info("finish processing on %f[sec].", stats[PROCESSING_TIME])
         logging.info(
@@ -195,13 +216,15 @@ class Streamer(object):
             stats[PROCESSING_FAILURE])
         return stats
 
-# XXX: Import simple map-reduce script.
 # `http://www.doughellmann.com/PyMOTW/multiprocessing/mapreduce.html`
 
 
 class CliHandler(object):
 
     """ Simple command line arguments handler.
+
+    :param streamer: streaming object
+    :type streamer: Streamer
     """
 
     def __init__(self, streamer):
@@ -210,6 +233,12 @@ class CliHandler(object):
     def reader(self, fp, encoding=None):
         """ Simple `open` wrapper for several file types.
         This supports ``.gz`` and ``.json``.
+
+        :param fp: opened file
+        :type fp: file pointer
+        :param encoding: encoding of opened file
+        :type encoding: string
+        :rtype: file pointer
         """
         _, suffix = os.path.splitext(fp.name)
         if suffix == '.gz':
@@ -219,22 +248,26 @@ class CliHandler(object):
             return json.load(fp)
         return fp
 
-    def handle(self, files, encoding):
-        """
-        :param files: target files.
-        :type files: list of file pointers.
-        :rtype: list of statistics.
+    def handle(self, files, encoding, chunksize=1):
+        """ Handle given files with given encoding.
+
+        :param files: opened files.
+        :type files: list
+        :param encoding: encoding of opened file
+        :type encoding: string
+        :param chunksize: a number of chunk
+        :type chunksize: int
+        :rtype: list
         """
         stats = []
         try:
             if files:
                 logging.info("Input file count: %d", len(files))
-                # XXX: enable multiprocessing support
                 for fp in files:
                     fname = fp.name
                     logging.info("Start processing: %s", fname)
                     reader = self.reader(fp, encoding)
-                    parsed = self.streamer.consume(reader)
+                    parsed = self.streamer.consume(reader, chunksize)
                     logging.info("End processing: %s", fname)
                     parsed['file'] = fname
                     stats.append(parsed)
@@ -254,21 +287,24 @@ class CsvHandler(CliHandler):
 
 
 def clistream(Handler, reporter, *args, **kwargs):
-    """ Handle stream data on command line interface.
+    """ Handle stream data on command line interface,
+    and returns statistics of success, error, and total amount.
 
     :param Handler:
     :param reporter: callback to report processed value
     :type reporter: callable
     :param kwargs: keywords, including ``files`` and ``input_encoding``.
-    :rtype: statistics of success, error, and total amount.
+    :rtype: list
     """
     # Follow the rule of `parse_arguments()`
     files = kwargs.get('files')
     encoding = kwargs.get('input_encoding')
+    processes = kwargs.get('processes')
+    chunksize = kwargs.get('chunksize')
 
-    s = Streamer(reporter, *args)
+    s = Streamer(reporter, processes=processes, *args)
     handler = Handler(s)
 
-    return handler.handle(files, encoding)
+    return handler.handle(files, encoding, chunksize)
 
 # vim: set et ts=4 sw=4 cindent fileencoding=utf-8 :
